@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -219,7 +219,7 @@ client_data_new (gchar *sender)
 			           error ? error->message : "no error given");
 			g_clear_error (&error);
 			g_free (filename);
-			return FALSE;
+			return cd;
 		}
 
 		g_free (filename);
@@ -301,21 +301,18 @@ client_get_for_context (DBusGMethodInvocation *context)
 
 	if (G_UNLIKELY (!clients)) {
 		clients_init ();
+	}
 
+	cd = g_hash_table_lookup (clients, sender);
+	if (!cd) {
 		cd = client_data_new (sender);
 		g_hash_table_insert (clients, sender, cd);
 	} else {
-		cd = g_hash_table_lookup (clients, sender);
-
-		if (G_UNLIKELY (!cd)) {
-			cd = client_data_new (sender);
-			g_hash_table_insert (clients, sender, cd);
-		} else {
-			g_get_current_time (&cd->last_time);
-			g_free (sender);
-		}
+		g_free (sender);
 	}
 
+	g_get_current_time (&cd->last_time);
+
 	return cd;
 }
 
