@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,16 +34,7 @@ G_BEGIN_DECLS
 gchar **    tracker_dbus_query_result_to_strv            (TrackerDBResultSet *result_set,
                                                           gint                column,
                                                           gint               *count);
-gchar **    tracker_dbus_query_result_numeric_to_strv    (TrackerDBResultSet *result_set,
-                                                          gint                column,
-                                                          gint               *count);
-gchar **    tracker_dbus_query_result_columns_to_strv    (TrackerDBResultSet *result_set,
-                                                          gint                offset_column,
-                                                          gint                until_column,
-                                                          gboolean            rewind);
-GHashTable *tracker_dbus_query_result_to_hash_table      (TrackerDBResultSet *result_set);
 GPtrArray * tracker_dbus_query_result_to_ptr_array       (TrackerDBResultSet *result_set);
-GPtrArray * tracker_dbus_query_result_multi_to_ptr_array (TrackerDBResultSet *result_set);
 
 G_END_DECLS
 
