@@ -243,15 +243,18 @@ function_sparql_uri_is_parent (sqlite3_context *context,
 		return;
 	}
 
+	parent_len = sqlite3_value_bytes (argv[0]);
+
 	/* Check only one argument, it's going to
 	 * be compared with the other anyway.
 	 */
-	if (strstr (parent, "://") == NULL) {
-		sqlite3_result_int (context, FALSE);
-		return;
-	}
 
-	parent_len = strlen (parent);
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
 
 	/* Remove trailing '/', will
 	 * be checked later on uri.
@@ -312,15 +315,18 @@ function_sparql_uri_is_descendant (sqlite3_context *context,
 		return;
 	}
 
+	parent_len = sqlite3_value_bytes (argv[0]);
+
 	/* Check only one argument, it's going to
 	 * be compared with the other anyway.
 	 */
-	if (strstr (parent, "://") == NULL) {
-		sqlite3_result_int (context, FALSE);
-		return;
-	}
 
-	parent_len = strlen (parent);
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
 
 	/* Remove trailing '/', will
 	 * be checked later on uri.
