@@ -1,6 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2008 Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,9 +23,8 @@
 
 #include "tracker-db-interface.h"
 
-#define TRACKER_DB_RESULT_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_RESULT_SET, TrackerDBResultSetPrivate))
-
-typedef struct TrackerDBResultSetPrivate TrackerDBResultSetPrivate;
+#define TRACKER_DB_RESULT_SET_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_RESULT_SET, TrackerDBResultSetPrivate))
+#define TRACKER_DB_RESULT_SET_GET_PRIVATE(o) (((TrackerDBResultSet*)o)->priv)
 
 struct TrackerDBResultSetPrivate {
 	GType *col_types;
@@ -114,42 +112,6 @@ tracker_db_cursor_get_type (void)
 }
 
 
-/* Boxed type for blobs */
-static gpointer
-blob_copy (gpointer boxed)
-{
-	GByteArray *array, *copy;
-
-	array = (GByteArray *) boxed;
-	copy = g_byte_array_sized_new (array->len);
-	g_byte_array_append (copy, array->data, array->len);
-
-	return copy;
-}
-
-static void
-blob_free (gpointer boxed)
-{
-	GByteArray *array;
-
-	array = (GByteArray *) boxed;
-	g_byte_array_free (array, TRUE);
-}
-
-GType
-tracker_db_blob_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		type = g_boxed_type_register_static ("TrackerDBBlob",
-		                                     blob_copy,
-		                                     blob_free);
-	}
-
-	return type;
-}
-
 /* TrackerDBResultSet */
 static void
 tracker_db_result_set_set_property (GObject       *object,
@@ -251,6 +213,7 @@ tracker_db_result_set_class_init (TrackerDBResultSetClass *class)
 static void
 tracker_db_result_set_init (TrackerDBResultSet *result_set)
 {
+	result_set->priv = TRACKER_DB_RESULT_SET_GET_PRIVATE_O (result_set);
 }
 
 static TrackerDBResultSet *
@@ -280,7 +243,10 @@ tracker_db_interface_create_statement (TrackerDBInterface  *interface,
 	va_list args;
 	gchar *str;
 
-	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (interface), NULL);
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (interface), NULL); */
+
+	g_return_val_if_fail (interface != NULL, NULL);
 	g_return_val_if_fail (query != NULL, NULL);
 
 	va_start (args, query);
@@ -365,7 +331,7 @@ tracker_db_interface_start_transaction (TrackerDBInterface *interface)
 }
 
 gboolean
-tracker_db_interface_end_transaction (TrackerDBInterface *interface)
+tracker_db_interface_end_db_transaction (TrackerDBInterface *interface)
 {
 	gboolean in_transaction;
 	GError *error = NULL;
@@ -391,28 +357,16 @@ tracker_db_interface_end_transaction (TrackerDBInterface *interface)
 	return TRUE;
 }
 
-void
-tracker_db_interface_disconnect (TrackerDBInterface  *interface)
-{
-	g_return_if_fail (TRACKER_IS_DB_INTERFACE (interface));
-
-	TRACKER_DB_INTERFACE_GET_IFACE (interface)->disconnect (interface);
-}
-
-void
-tracker_db_interface_reconnect (TrackerDBInterface  *interface)
-{
-	g_return_if_fail (TRACKER_IS_DB_INTERFACE (interface));
-
-	TRACKER_DB_INTERFACE_GET_IFACE (interface)->reconnect (interface);
-}
 
 void
 tracker_db_statement_bind_double (TrackerDBStatement    *stmt,
                                   int                    idx,
                                   double                 value)
 {
-	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_if_fail (stmt != NULL);
 
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_double (stmt, idx, value);
 }
@@ -422,18 +376,23 @@ tracker_db_statement_bind_int (TrackerDBStatement       *stmt,
                                int                       idx,
                                int                       value)
 {
-	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+	/* Removed for performance:
+	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt)); */
+
+	g_return_if_fail (stmt != NULL);
 
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_int (stmt, idx, value);
 }
 
-
 void
 tracker_db_statement_bind_int64 (TrackerDBStatement     *stmt,
                                  int                     idx,
                                  gint64                          value)
 {
-	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_if_fail (stmt != NULL);
 
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_int64 (stmt, idx, value);
 }
@@ -442,7 +401,10 @@ void
 tracker_db_statement_bind_null (TrackerDBStatement      *stmt,
                                 int                      idx)
 {
-	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_if_fail (stmt != NULL);
 
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_null (stmt, idx);
 }
@@ -452,7 +414,10 @@ tracker_db_statement_bind_text (TrackerDBStatement      *stmt,
                                 int                      idx,
                                 const gchar             *value)
 {
-	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_if_fail (stmt != NULL);
 
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_text (stmt, idx, value);
 }
@@ -463,7 +428,10 @@ tracker_db_statement_execute (TrackerDBStatement         *stmt,
 {
 	TrackerDBResultSet *result_set;
 
-	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL);
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_val_if_fail (stmt != NULL, NULL); 
 
 	result_set = TRACKER_DB_STATEMENT_GET_IFACE (stmt)->execute (stmt, error);
 
@@ -474,7 +442,10 @@ TrackerDBCursor *
 tracker_db_statement_start_cursor (TrackerDBStatement    *stmt,
                                    GError               **error)
 {
-	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL);
+	/* Removed for performance
+	g_return_val_if_fail (TRACKER_IS_DB_STATEMENT (stmt), NULL); */
+
+	g_return_val_if_fail (stmt != NULL, NULL);
 
 	return TRACKER_DB_STATEMENT_GET_IFACE (stmt)->start_cursor (stmt, error);
 }
@@ -492,7 +463,10 @@ tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 gboolean
 tracker_db_cursor_iter_next (TrackerDBCursor *cursor)
 {
-	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), FALSE);
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), FALSE); */
+
+	g_return_val_if_fail (cursor != NULL, FALSE);
 
 	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->iter_next (cursor);
 }
@@ -508,7 +482,10 @@ tracker_db_cursor_get_n_columns (TrackerDBCursor *cursor)
 void
 tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *value)
 {
-	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor));
+	/* Removed for performance 
+	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor)); */
+
+	g_return_if_fail (cursor != NULL);
 
 	TRACKER_DB_CURSOR_GET_IFACE (cursor)->get_value (cursor, column, value);
 }
@@ -516,7 +493,10 @@ tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *val
 const gchar*
 tracker_db_cursor_get_string (TrackerDBCursor *cursor, guint            column)
 {
-	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), NULL);
+	/* Removed for performance 
+	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), NULL); */
+
+	g_return_val_if_fail (cursor != NULL, NULL);
 
 	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->get_string (cursor, column);
 }
@@ -524,7 +504,10 @@ tracker_db_cursor_get_string (TrackerDBCursor *cursor, guint            column)
 gint
 tracker_db_cursor_get_int (TrackerDBCursor *cursor, guint            column)
 {
-	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), -1);
+	/* Removed for performance
+	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), -1); */
+
+	g_return_val_if_fail (cursor != NULL, -1);
 
 	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->get_int (cursor, column);
 }
@@ -532,7 +515,10 @@ tracker_db_cursor_get_int (TrackerDBCursor *cursor, guint            column)
 gdouble
 tracker_db_cursor_get_double (TrackerDBCursor *cursor, guint            column)
 {
-	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), -1);
+	/* Removed for performance
+	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), -1); */
+
+	g_return_val_if_fail (cursor != NULL, -1);
 
 	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->get_double (cursor, column);
 }
