@@ -42,7 +42,7 @@ typedef enum {
 typedef enum {
 	TRACKER_DB_MANAGER_FORCE_REINDEX    = 1 << 1,
 	TRACKER_DB_MANAGER_REMOVE_CACHE     = 1 << 2,
-	TRACKER_DB_MANAGER_LOW_MEMORY_MODE  = 1 << 3,
+	/* 1 << 3 Was low mem mode */
 	TRACKER_DB_MANAGER_REMOVE_ALL       = 1 << 4,
 	TRACKER_DB_MANAGER_READONLY         = 1 << 5
 } TrackerDBManagerFlags;
@@ -65,6 +65,9 @@ gboolean            tracker_db_manager_has_enough_space  (void);
 TrackerDBManagerFlags
                     tracker_db_manager_get_flags         (void);
 
+gboolean            tracker_db_manager_get_first_index_done (void);
+void                tracker_db_manager_set_first_index_done (gboolean done);
+
 G_END_DECLS
 
 #endif /* __LIBTRACKER_DB_MANAGER_H__ */
