@@ -1,35 +1,29 @@
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
- *
- * Author: Philip Van Hoof <philip@codeminded.be>
  */
 
 #include "config.h"
 
-#include <locale.h>
 #include <string.h>
 #include <ctype.h>
-#include <glib.h>
-
-#include <libtracker-common/tracker-type-utils.h>
-#include <libtracker-common/tracker-utils.h>
 
 #include "tracker-exif.h"
+#include "tracker-utils.h"
 
 #ifdef HAVE_LIBEXIF
 
@@ -37,13 +31,12 @@
 
 #define EXIF_DATE_FORMAT "%Y:%m:%d %H:%M:%S"
 
-#ifndef HAVE_STRCASESTR
-
 /**
  * SECTION:tracker-exif
+ * @title: Exif
  * @short_description: Exchangeable Image File Format (EXIF)
  * @stability: Stable
- * @include: libtracker-extract/tracker-exif.h
+ * @include: libtracker-extract/tracker-extract.h
  *
  * Exchangeable Image File Format (EXIF) is a specification for the
  * image file format used by digital cameras. The specification uses
@@ -55,34 +48,6 @@
  * using these standards.
  **/
 
-static gchar *
-strcasestr (const gchar *haystack,
-            const gchar *needle)
-{
-	gchar *p;
-	gchar *startn = NULL;
-	gchar *np = NULL;
-
-	for (p = (gchar *) haystack; *p; p++) {
-		if (np) {
-			if (toupper (*p) == toupper (*np)) {
-				if (!*++np) {
-					return startn;
-				}
-			} else {
-				np = 0;
-			}
-		} else if (toupper (*p) == toupper (*needle)) {
-			np = (gchar *) needle + 1;
-			startn = p;
-		}
-	}
-
-	return NULL;
-}
-
-#endif /* HAVE_STRCASESTR */
-
 static gchar *
 get_date (ExifData *exif, 
           ExifTag   tag)
@@ -123,21 +88,29 @@ get_focal_length (ExifData *exif,
 }
 
 static gchar *
-get_flash (ExifData *exif, 
+get_flash (ExifData *exif,
            ExifTag   tag)
 {
 	ExifEntry *entry = exif_data_get_entry (exif, tag);
 
 	if (entry) {
-		gchar buf[1024];
+		ExifByteOrder order;
+		gushort flash;
 
-		exif_entry_get_value (entry, buf, 1024);
+		order = exif_data_get_byte_order (exif);
+		flash = exif_get_short (entry->data, order);
 
-		if (strcasestr (buf, "flash fired")) {
+		switch (flash) {
+		case 0x0000: /* No flash */
+		case 0x0005: /* Without strobe */
+		case 0x0008: /* Flash did not fire */
+		case 0x0010: /* Flash in compulsory mode, did not fire */
+		case 0x0018: /* Flash in auto mode, did not fire */
+		case 0x0058: /* Only red-eye reduction mode */
+			return g_strdup ("nmm:flash-off");
+		default:
 			return g_strdup ("nmm:flash-on");
 		}
-
-		return g_strdup ("nmm:flash-off");
 	}
 
 	return NULL;
@@ -216,28 +189,32 @@ get_orientation (ExifData *exif,
 	ExifEntry *entry = exif_data_get_entry (exif, tag);
 
 	if (entry) {
-		gchar buf[1024];
+		ExifByteOrder order;
+		gushort orientation;
 
-		exif_entry_get_value (entry, buf, 1024);
+		order = exif_data_get_byte_order (exif);
+		orientation = exif_get_short (entry->data, order);
 
-		if (g_ascii_strcasecmp (buf, "top - left") == 0)
+		switch (orientation) {
+		case 1:
 			return g_strdup ("nfo:orientation-top");
-		else if (g_ascii_strcasecmp (buf, "top - right") == 0)
+		case 2:
 			return g_strdup ("nfo:orientation-top-mirror");
-		else if (g_ascii_strcasecmp (buf, "bottom - right") == 0)
+		case 3:
 			return g_strdup ("nfo:orientation-bottom");
-		else if (g_ascii_strcasecmp (buf, "bottom - left") == 0)
+		case 4:
 			return g_strdup ("nfo:orientation-bottom-mirror");
-		else if (g_ascii_strcasecmp (buf, "left - top") == 0)
+		case 5:
 			return g_strdup ("nfo:orientation-left-mirror");
-		else if (g_ascii_strcasecmp (buf, "right - top") == 0)
+		case 6:
 			return g_strdup ("nfo:orientation-right");
-		else if (g_ascii_strcasecmp (buf, "right - bottom") == 0)
+		case 7:
 			return g_strdup ("nfo:orientation-right-mirror");
-		else if (g_ascii_strcasecmp (buf, "left - bottom") == 0)
+		case 8:
 			return g_strdup ("nfo:orientation-left");
-
-		return g_strdup ("nfo:orientation-top");
+		default:
+			return g_strdup ("nfo:orientation-top");
+		}
 	}
 
 	return NULL;
@@ -250,24 +227,28 @@ get_metering_mode (ExifData *exif,
 	ExifEntry *entry = exif_data_get_entry (exif, tag);
 
 	if (entry) {
-		gchar buf[1024];
+		ExifByteOrder order;
+		gushort metering;
 
-		exif_entry_get_value (entry, buf, 1024);
+		order = exif_data_get_byte_order (exif);
+		metering = exif_get_short (entry->data, order);
 
-		if (strcasestr (buf, "center"))
-			return g_strdup ("nmm:metering-mode-center-weighted-average");
-		else if (strcasestr (buf, "average"))
+		switch (metering) {
+		case 1:
 			return g_strdup ("nmm:metering-mode-average");
-		else if (strcasestr (buf, "spot"))
+		case 2:
+			return g_strdup ("nmm:metering-mode-center-weighted-average");
+		case 3:
 			return g_strdup ("nmm:metering-mode-spot");
-		else if (strcasestr (buf, "multispot"))
+		case 4:
 			return g_strdup ("nmm:metering-mode-multispot");
-		else if (strcasestr (buf, "pattern"))
+		case 5:
 			return g_strdup ("nmm:metering-mode-pattern");
-		else if (strcasestr (buf, "partial"))
+		case 6:
 			return g_strdup ("nmm:metering-mode-partial");
-		else
+		default:
 			return g_strdup ("nmm:metering-mode-other");
+		}
 	}
 
 	return NULL;
@@ -280,16 +261,17 @@ get_white_balance (ExifData *exif,
 	ExifEntry *entry = exif_data_get_entry (exif, tag);
 
 	if (entry) {
-		gchar buf[1024];
+		ExifByteOrder order;
+		gushort white_balance;
 
-		exif_entry_get_value (entry, buf, 1024);
+		order = exif_data_get_byte_order (exif);
+		white_balance = exif_get_short (entry->data, order);
 
-		if (strcasestr (buf, "auto"))
+                if (white_balance == 0)
 			return g_strdup ("nmm:white-balance-auto");
 
 		/* Found in the field: sunny, fluorescent, incandescent, cloudy.
 		 * These will this way also yield as manual. */
-
 		return g_strdup ("nmm:white-balance-manual");
 	}
 
@@ -337,6 +319,10 @@ tracker_exif_read (const unsigned char *buffer,
                    const gchar         *uri,
                    TrackerExifData     *data)
 {
+#ifdef HAVE_LIBEXIF
+	ExifData *exif;
+#endif
+
 	g_return_val_if_fail (buffer != NULL, FALSE);
 	g_return_val_if_fail (len > 0, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
@@ -345,7 +331,6 @@ tracker_exif_read (const unsigned char *buffer,
 	memset (data, 0, sizeof (TrackerExifData));
 
 #ifdef HAVE_LIBEXIF
-	ExifData *exif;
 
 	exif = exif_data_new ();
 
@@ -398,8 +383,20 @@ tracker_exif_read (const unsigned char *buffer,
 		data->metering_mode = get_metering_mode (exif, EXIF_TAG_METERING_MODE);
 	if (!data->white_balance)
 		data->white_balance = get_white_balance (exif, EXIF_TAG_WHITE_BALANCE);
-	if (!data->copyright)
+	if (!data->copyright) {
+		gchar *strip_off;
 		data->copyright = get_value (exif, EXIF_TAG_COPYRIGHT);
+		/* exiftool catenates this to the string, so we don't need it */
+		if (data->copyright) {
+			strip_off = strstr (data->copyright, " (Photographer) -  (Editor)");
+			if (strip_off) {
+				*strip_off = '\0';
+			}
+		}
+	}
+	if (!data->software)
+		data->software = get_value (exif, EXIF_TAG_SOFTWARE);
+
 
 	exif_data_free (exif);
 #endif /* HAVE_LIBEXIF */
