@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -103,6 +103,7 @@ typedef struct {
 	TrackerCrawler *crawler;
 	DirectoryRootInfo  *root_info;
 	DirectoryProcessingData *dir_info;
+	GFile *dir_file;
 } EnumeratorData;
 
 static void     crawler_finalize        (GObject         *object);
@@ -263,9 +264,18 @@ check_file (TrackerCrawler    *crawler,
             GFile             *file)
 {
 	gboolean use = FALSE;
+	TrackerCrawlerPrivate *priv;
+
+	priv = TRACKER_CRAWLER_GET_PRIVATE (crawler);
 
 	g_signal_emit (crawler, signals[CHECK_FILE], 0, file, &use);
 
+	/* Crawler may have been stopped while waiting for the 'use' value,
+	 * and the DirectoryRootInfo already disposed... */
+	if (!priv->is_running) {
+		return FALSE;
+	}
+
 	info->files_found++;
 
 	if (!use) {
@@ -278,12 +288,21 @@ check_file (TrackerCrawler    *crawler,
 static gboolean
 check_directory (TrackerCrawler    *crawler,
 		 DirectoryRootInfo *info,
-                 GFile             *file)
+		 GFile             *file)
 {
 	gboolean use = FALSE;
+	TrackerCrawlerPrivate *priv;
+
+	priv = TRACKER_CRAWLER_GET_PRIVATE (crawler);
 
 	g_signal_emit (crawler, signals[CHECK_DIRECTORY], 0, file, &use);
 
+	/* Crawler may have been stopped while waiting for the 'use' value,
+	 * and the DirectoryRootInfo already disposed... */
+	if (!priv->is_running) {
+		return FALSE;
+	}
+
 	info->directories_found++;
 
 	if (!use) {
@@ -436,7 +455,10 @@ process_func (gpointer data)
 
 			dir_data->was_inspected = TRUE;
 
-			if (iterate) {
+			/* Crawler may have been already stopped while we were waiting for the
+			 *  check_directory return value, and thus we should check if it's
+			 *  running before going on with the iteration */
+			if (priv->is_running && iterate) {
 				/* Directory contents haven't been inspected yet,
 				 * stop this idle function while it's being iterated
 				 */
@@ -456,15 +478,19 @@ process_func (gpointer data)
 			child_data = dir_data->children->data;
 			dir_data->children = g_slist_remove (dir_data->children, child_data);
 
-			if ((child_data->is_dir &&
-			     check_directory (crawler, info, child_data->child)) ||
-			    (!child_data->is_dir &&
-			     check_file (crawler, info, child_data->child))) {
+			if (((child_data->is_dir &&
+			      check_directory (crawler, info, child_data->child)) ||
+			     (!child_data->is_dir &&
+			      check_file (crawler, info, child_data->child))) &&
+			    /* Crawler may have been already stopped while we were waiting for the
+			     *	check_directory or check_file return value, and thus we should
+			     *	 check if it's running before going on */
+			    priv->is_running) {
 				child_node = g_node_prepend_data (dir_data->node,
 								  g_object_ref (child_data->child));
 			}
 
-			if (child_node && child_data->is_dir) {
+			if (priv->is_running && child_node && child_data->is_dir) {
 				DirectoryProcessingData *child_dir_data;
 
 				child_dir_data = directory_processing_data_new (child_node);
@@ -547,6 +573,9 @@ enumerator_data_new (TrackerCrawler          *crawler,
 	ed->crawler = g_object_ref (crawler);
 	ed->root_info = root_info;
 	ed->dir_info = dir_info;
+	/* Make sure there's always a ref of the GFile while we're
+	 * iterating it */
+	ed->dir_file = g_object_ref (G_FILE (dir_info->node->data));
 
 	return ed;
 }
@@ -581,6 +610,7 @@ enumerator_data_process (EnumeratorData *ed)
 static void
 enumerator_data_free (EnumeratorData *ed)
 {
+	g_object_unref (ed->dir_file);
 	g_object_unref (ed->crawler);
 	g_slice_free (EnumeratorData, ed);
 }
@@ -745,12 +775,10 @@ file_enumerate_children (TrackerCrawler          *crawler,
 			 DirectoryProcessingData *dir_data)
 {
 	EnumeratorData *ed;
-	GFile *file;
 
-	file = dir_data->node->data;
 	ed = enumerator_data_new (crawler, info, dir_data);
 
-	g_file_enumerate_children_async (file,
+	g_file_enumerate_children_async (ed->dir_file,
 	                                 FILE_ATTRIBUTES,
 	                                 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 	                                 G_PRIORITY_LOW,
@@ -790,6 +818,10 @@ tracker_crawler_start (TrackerCrawler *crawler,
 
 	priv->timer = g_timer_new ();
 
+	if (priv->is_paused) {
+		g_timer_stop (priv->timer);
+	}
+
 	/* Set as running now */
 	priv->is_running = TRUE;
 	priv->is_finished = FALSE;
@@ -811,6 +843,11 @@ tracker_crawler_stop (TrackerCrawler *crawler)
 
 	priv = crawler->private;
 
+	/* If already not running, just ignore */
+	if (!priv->is_running) {
+		return;
+	}
+
 	priv->is_running = FALSE;
 	g_cancellable_cancel (priv->cancellable);
 
