@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -45,7 +45,6 @@
 #define DEFAULT_THROTTLE                         0        /* 0->20 */
 #define DEFAULT_SCAN_TIMEOUT                     0        /* 0->1000 */
 #define DEFAULT_CACHE_TIMEOUT                    60       /* 0->1000 */
-#define DEFAULT_INDEX_MOUNTED_DIRECTORIES        TRUE
 #define DEFAULT_INDEX_REMOVABLE_DEVICES          TRUE
 #define DEFAULT_INDEX_OPTICAL_DISCS              FALSE
 #define DEFAULT_INDEX_ON_BATTERY                 FALSE
@@ -66,12 +65,13 @@ typedef struct {
 	gint      throttle;
 	gboolean  index_on_battery;
 	gboolean  index_on_battery_first_time;
-	gboolean  index_mounted_directories;
 	gboolean  index_removable_devices;
 	gboolean  index_optical_discs;
 	gint      low_disk_space_limit;
 	GSList   *index_recursive_directories;
+	GSList	 *index_recursive_directories_unfiltered;
 	GSList   *index_single_directories;
+	GSList	 *index_single_directories_unfiltered;
 	GSList   *ignored_directories;
 	GSList   *ignored_directories_with_content;
 	GSList   *ignored_files;
@@ -124,7 +124,6 @@ enum {
 	PROP_THROTTLE,
 	PROP_INDEX_ON_BATTERY,
 	PROP_INDEX_ON_BATTERY_FIRST_TIME,
-	PROP_INDEX_MOUNTED_DIRECTORIES,
 	PROP_INDEX_REMOVABLE_DEVICES,
 	PROP_INDEX_OPTICAL_DISCS,
 	PROP_LOW_DISK_SPACE_LIMIT,
@@ -147,7 +146,6 @@ static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_INT,     "throttle",                         GROUP_INDEXING, "Throttle"                  },
 	{ G_TYPE_BOOLEAN, "index-on-battery",                 GROUP_INDEXING, "IndexOnBattery"            },
 	{ G_TYPE_BOOLEAN, "index-on-battery-first-time",      GROUP_INDEXING, "IndexOnBatteryFirstTime"   },
-	{ G_TYPE_BOOLEAN, "index-mounted-directories",        GROUP_INDEXING, "IndexMountedDirectories"   },
 	{ G_TYPE_BOOLEAN, "index-removable-devices",          GROUP_INDEXING, "IndexRemovableMedia"       },
 	{ G_TYPE_BOOLEAN, "index-optical-discs",              GROUP_INDEXING, "IndexOpticalDiscs"         },
 	{ G_TYPE_INT,     "low-disk-space-limit",             GROUP_INDEXING, "LowDiskSpaceLimit"         },
@@ -244,14 +242,6 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                       " Set to true to index while running on battery for the first time only",
 	                                                       DEFAULT_INDEX_ON_BATTERY_FIRST_TIME,
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (object_class,
-	                                 PROP_INDEX_MOUNTED_DIRECTORIES,
-	                                 g_param_spec_boolean ("index-mounted-directories",
-	                                                       "Index mounted directories",
-	                                                       " Set to true to enable traversing mounted directories on other file systems\n"
-	                                                       " (this excludes removable devices)",
-	                                                       DEFAULT_INDEX_MOUNTED_DIRECTORIES,
-	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 	                                 PROP_INDEX_REMOVABLE_DEVICES,
 	                                 g_param_spec_boolean ("index-removable-devices",
@@ -372,10 +362,6 @@ config_set_property (GObject      *object,
 		tracker_config_set_index_on_battery_first_time (TRACKER_CONFIG (object),
 		                                                g_value_get_boolean (value));
 		break;
-	case PROP_INDEX_MOUNTED_DIRECTORIES:
-		tracker_config_set_index_mounted_directories (TRACKER_CONFIG (object),
-		                                              g_value_get_boolean (value));
-		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		tracker_config_set_index_removable_devices (TRACKER_CONFIG (object),
 		                                            g_value_get_boolean (value));
@@ -455,9 +441,6 @@ config_get_property (GObject    *object,
 	case PROP_INDEX_ON_BATTERY_FIRST_TIME:
 		g_value_set_boolean (value, priv->index_on_battery_first_time);
 		break;
-	case PROP_INDEX_MOUNTED_DIRECTORIES:
-		g_value_set_boolean (value, priv->index_mounted_directories);
-		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		g_value_set_boolean (value, priv->index_removable_devices);
 		break;
@@ -528,9 +511,15 @@ config_finalize (GObject *object)
 	g_slist_foreach (priv->index_single_directories, (GFunc) g_free, NULL);
 	g_slist_free (priv->index_single_directories);
 
+	g_slist_foreach (priv->index_single_directories_unfiltered, (GFunc) g_free, NULL);
+	g_slist_free (priv->index_single_directories_unfiltered);
+
 	g_slist_foreach (priv->index_recursive_directories, (GFunc) g_free, NULL);
 	g_slist_free (priv->index_recursive_directories);
 
+	g_slist_foreach (priv->index_recursive_directories_unfiltered, (GFunc) g_free, NULL);
+	g_slist_free (priv->index_recursive_directories_unfiltered);
+
 	(G_OBJECT_CLASS (tracker_config_parent_class)->finalize) (object);
 }
 
@@ -844,7 +833,6 @@ config_load (TrackerConfig *config,
 
 	for (i = 0; i < G_N_ELEMENTS (conversions); i++) {
 		gboolean has_key;
-		gboolean is_directory_list;
 
 		has_key = g_key_file_has_key (file->key_file,
 		                              conversions[i].group,
@@ -870,82 +858,69 @@ config_load (TrackerConfig *config,
 
 		case G_TYPE_POINTER: {
 			GSList *new_dirs, *old_dirs, *l;
-			gboolean check_for_duplicates = FALSE;
-                        gboolean is_recursive = TRUE;
-                        gboolean equal;
+			gboolean is_recursive = TRUE;
+			gboolean equal;
 
-			is_directory_list = TRUE;
+			if (strcmp (conversions[i].property, "index-recursive-directories") == 0 ||
+			    strcmp (conversions[i].property, "index-single-directories") == 0 ||
+			    strcmp (conversions[i].property, "ignored-directories") == 0) {
+				is_recursive = strcmp (conversions[i].property, "index-recursive-directories") == 0;
+				tracker_keyfile_object_load_directory_list (G_OBJECT (file),
+				                                            conversions[i].property,
+				                                            file->key_file,
+				                                            conversions[i].group,
+				                                            conversions[i].key,
+				                                            is_recursive,
+				                                            &new_dirs);
 
-			if (strcmp (conversions[i].property, "index-recursive-directories") != 0 &&
-			    strcmp (conversions[i].property, "index-single-directories") != 0) {
-                                tracker_keyfile_object_load_string_list (G_OBJECT (file),
-                                                                         conversions[i].property,
-                                                                         file->key_file,
-                                                                         conversions[i].group,
-                                                                         conversions[i].key,
-                                                                         NULL);
-                                continue;
+				for (l = new_dirs; l; l = l->next) {
+					const gchar *path_to_use;
+
+					/* Must be a special dir */
+					if (strcmp (l->data, "&DESKTOP") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+					} else if (strcmp (l->data, "&DOCUMENTS") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS);
+					} else if (strcmp (l->data, "&DOWNLOAD") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
+					} else if (strcmp (l->data, "&MUSIC") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
+					} else if (strcmp (l->data, "&PICTURES") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+					} else if (strcmp (l->data, "&PUBLIC_SHARE") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE);
+					} else if (strcmp (l->data, "&TEMPLATES") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES);
+					} else if (strcmp (l->data, "&VIDEOS") == 0) {
+						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
+					} else {
+						path_to_use = NULL;
+					}
+
+					if (path_to_use) {
+						g_free (l->data);
+						l->data = g_strdup (path_to_use);
+					}
+				}
+			} else {
+				tracker_keyfile_object_load_string_list (G_OBJECT (file),
+				                                         conversions[i].property,
+				                                         file->key_file,
+				                                         conversions[i].group,
+				                                         conversions[i].key,
+				                                         &new_dirs);
 			}
 
-                        is_recursive = strcmp (conversions[i].property, "index-recursive-directories") == 0;
-                        tracker_keyfile_object_load_directory_list (G_OBJECT (file),
-                                                                    conversions[i].property,
-                                                                    file->key_file,
-                                                                    conversions[i].group,
-                                                                    conversions[i].key,
-                                                                    is_recursive,
-                                                                    &new_dirs);
-                        g_object_get (config, conversions[i].property, &old_dirs, NULL);
+			g_object_get (config, conversions[i].property, &old_dirs, NULL);
 
-			for (l = new_dirs; l; l = l->next) {
-				const gchar *path_to_use;
+			equal = tracker_gslist_with_string_data_equal (new_dirs, old_dirs);
 
-				/* Must be a special dir */
-				if (strcmp (l->data, "&DESKTOP") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
-				} else if (strcmp (l->data, "&DOCUMENTS") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS);
-				} else if (strcmp (l->data, "&DOWNLOAD") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
-				} else if (strcmp (l->data, "&MUSIC") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
-				} else if (strcmp (l->data, "&PICTURES") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
-				} else if (strcmp (l->data, "&PUBLIC_SHARE") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE);
-				} else if (strcmp (l->data, "&TEMPLATES") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES);
-				} else if (strcmp (l->data, "&VIDEOS") == 0) {
-					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
-				} else {
-					path_to_use = NULL;
-				}
-
-				if (path_to_use) {
-					check_for_duplicates = TRUE;
-					g_free (l->data);
-					l->data = g_strdup (path_to_use);
-				}
-			}
-
-			if (check_for_duplicates) {
-				GSList *filtered;
-
-				filtered = tracker_path_list_filter_duplicates (new_dirs, ".", is_recursive);
-				g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
-				g_slist_free (new_dirs);
-
-                                new_dirs = filtered;
+			if (!equal) {
+				g_object_set (config, conversions[i].property, new_dirs, NULL);
                         }
 
-                        equal = tracker_gslist_with_string_data_equal (new_dirs, old_dirs);
-
-                        if (!equal) {
-                                g_object_set (config, conversions[i].property, new_dirs, NULL);
-                        }
-
-                        g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
-                        g_slist_free (new_dirs);
+			g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
+			g_slist_free (new_dirs);
 
 			break;
 		}
@@ -998,7 +973,6 @@ config_save (TrackerConfig *config)
 				g_object_get (config, conversions[i].property, &dirs, NULL);
 
 				for (l = dirs; l; l = l->next) {
-					const gchar *dir;
 					const gchar *path_to_use;
 
 					/* FIXME: This doesn't work
@@ -1008,9 +982,12 @@ config_save (TrackerConfig *config)
 					 * breaks. Need a better
 					 * solution at some point.
 					 */
-
-					dir = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
-					if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) {
+					if (g_strcmp0 (l->data, g_get_home_dir ()) == 0) {
+						/* Home dir gets straight into configuration,
+						 * regardless of having XDG dirs pointing to it.
+						 */
+						path_to_use = "$HOME";
+					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) {
 						path_to_use = "&DESKTOP";
 					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS)) == 0) {
 						path_to_use = "&DOCUMENTS";
@@ -1170,18 +1147,6 @@ tracker_config_get_index_on_battery_first_time (TrackerConfig *config)
 	return priv->index_on_battery_first_time;
 }
 
-gboolean
-tracker_config_get_index_mounted_directories (TrackerConfig *config)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_INDEX_MOUNTED_DIRECTORIES);
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	return priv->index_mounted_directories;
-}
-
 gboolean
 tracker_config_get_index_removable_devices (TrackerConfig *config)
 {
@@ -1230,6 +1195,18 @@ tracker_config_get_index_recursive_directories (TrackerConfig *config)
 	return priv->index_recursive_directories;
 }
 
+GSList *
+tracker_config_get_index_recursive_directories_unfiltered (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), NULL);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->index_recursive_directories_unfiltered;
+}
+
 GSList *
 tracker_config_get_index_single_directories (TrackerConfig *config)
 {
@@ -1242,6 +1219,18 @@ tracker_config_get_index_single_directories (TrackerConfig *config)
 	return priv->index_single_directories;
 }
 
+GSList *
+tracker_config_get_index_single_directories_unfiltered (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), NULL);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->index_single_directories_unfiltered;
+}
+
 GSList *
 tracker_config_get_ignored_directories (TrackerConfig *config)
 {
@@ -1410,20 +1399,6 @@ tracker_config_set_index_on_battery_first_time (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "index-on-battery-first-time");
 }
 
-void
-tracker_config_set_index_mounted_directories (TrackerConfig *config,
-                                              gboolean       value)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	priv->index_mounted_directories = value;
-	g_object_notify (G_OBJECT (config), "index-mounted-directories");
-}
-
 void
 tracker_config_set_index_removable_devices (TrackerConfig *config,
                                             gboolean       value)
@@ -1470,6 +1445,70 @@ tracker_config_set_low_disk_space_limit (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "low-disk-space-limit");
 }
 
+static void
+rebuild_filtered_lists (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+	GSList *old_list;
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	/* Filter single directories first, checking duplicates */
+	old_list = priv->index_single_directories;
+	priv->index_single_directories = NULL;
+
+	if (priv->index_single_directories_unfiltered) {
+		priv->index_single_directories =
+		        tracker_path_list_filter_duplicates (priv->index_single_directories_unfiltered,
+		                                             ".", FALSE);
+	}
+
+	if (!tracker_gslist_with_string_data_equal (old_list, priv->index_single_directories)) {
+		g_object_notify (G_OBJECT (config), "index-single-directories");
+	}
+
+	if (old_list) {
+		g_slist_foreach (old_list, (GFunc) g_free, NULL);
+		g_slist_free (old_list);
+	}
+
+	/* Filter recursive directories */
+	old_list = priv->index_recursive_directories;
+	priv->index_recursive_directories = NULL;
+
+	if (priv->index_recursive_directories_unfiltered) {
+		GSList *l, *new_list = NULL;
+
+		/* Remove elements already in single directories */
+		for (l = priv->index_recursive_directories_unfiltered; l; l = l->next) {
+			if (g_slist_find_custom (priv->index_single_directories,
+			                         l->data,
+			                         (GCompareFunc) g_strcmp0) != NULL) {
+				g_message ("Path '%s' being removed from recursive directories "
+				           "list, as it also exists in single directories list",
+				           (gchar *) l->data);
+			} else {
+				new_list = g_slist_prepend (new_list, l->data);
+			}
+		}
+
+		new_list = g_slist_reverse (new_list);
+
+		priv->index_recursive_directories =
+		        tracker_path_list_filter_duplicates (new_list, ".", TRUE);
+
+		g_slist_free (new_list);
+	}
+
+	if (!tracker_gslist_with_string_data_equal (old_list, priv->index_recursive_directories)) {
+		g_object_notify (G_OBJECT (config), "index-recursive-directories");
+	}
+
+	if (old_list) {
+		g_slist_foreach (old_list, (GFunc) g_free, NULL);
+		g_slist_free (old_list);
+	}
+}
 
 void
 tracker_config_set_index_recursive_directories (TrackerConfig *config,
@@ -1483,14 +1522,14 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 
 	priv = TRACKER_CONFIG_GET_PRIVATE (config);
 
-	l = priv->index_recursive_directories;
+	l = priv->index_recursive_directories_unfiltered;
 
         equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
-		priv->index_recursive_directories = NULL;
+		priv->index_recursive_directories_unfiltered = NULL;
 	} else {
-		priv->index_recursive_directories =
+		priv->index_recursive_directories_unfiltered =
 			tracker_gslist_copy_with_string_data (roots);
 	}
 
@@ -1501,7 +1540,7 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
                 return;
         }
 
-	g_object_notify (G_OBJECT (config), "index-recursive-directories");
+        rebuild_filtered_lists (config);
 }
 
 void
@@ -1516,14 +1555,14 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 
 	priv = TRACKER_CONFIG_GET_PRIVATE (config);
 
-	l = priv->index_single_directories;
+	l = priv->index_single_directories_unfiltered;
 
         equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
-		priv->index_single_directories = NULL;
+		priv->index_single_directories_unfiltered = NULL;
 	} else {
-		priv->index_single_directories =
+		priv->index_single_directories_unfiltered =
 			tracker_gslist_copy_with_string_data (roots);
 	}
 
@@ -1534,7 +1573,7 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
                 return;
         }
 
-	g_object_notify (G_OBJECT (config), "index-single-directories");
+        rebuild_filtered_lists (config);
 }
 
 void
