@@ -1,5 +1,5 @@
 /* Tracker - audio/video metadata extraction based on Xine
- * Copyright (C) 2006, Laurent Aguerreche (laurent.aguerreche@free.fr)
+ * Copyright (C) 2006, Laurent Aguerreche <laurent.aguerreche@free.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,20 +23,10 @@
 
 #include <glib.h>
 
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-utils.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
-
-#define NMM_PREFIX TRACKER_NMM_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define DC_PREFIX TRACKER_DC_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
 
 static void
 tracker_extract_xine (const gchar          *uri,
@@ -60,7 +50,7 @@ tracker_extract_xine (const gchar          *uri,
 	const char        *title;
 	const char        *author;
 	const char        *album;
-	const char        *year;
+	gchar             *year;
 	const char        *genre;
 	const char        *track;
 
@@ -248,10 +238,11 @@ tracker_extract_xine (const gchar          *uri,
 		tracker_sparql_builder_object_unvalidated (metadata, title);
 	}
 
-	year = xine_get_meta_info (stream, XINE_META_INFO_YEAR);
+	year = tracker_date_guess (xine_get_meta_info (stream, XINE_META_INFO_YEAR));
 	if (year) {
 		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
 		tracker_sparql_builder_object_unvalidated (metadata, year);
+		g_free (year);
 	}
 
 	genre = xine_get_meta_info (stream, XINE_META_INFO_GENRE);
