@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2006, Edward Duffy (eduffy@gmail.com)
- * Copyright (C) 2006, Laurent Aguerreche (laurent.aguerreche@free.fr)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Edward Duffy <eduffy@gmail.com>
+ * Copyright (C) 2006, Laurent Aguerreche <laurent.aguerreche@free.fr>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,21 +25,12 @@
 
 #include <glib.h>
 
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-os-dependant.h>
 #include <libtracker-common/tracker-utils.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
-#define NMM_PREFIX TRACKER_NMM_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define DC_PREFIX TRACKER_DC_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
 static void extract_mplayer (const gchar          *uri,
                              TrackerSparqlBuilder *preupdate,
                              TrackerSparqlBuilder *metadata);
@@ -82,7 +73,7 @@ static const gchar *info_tags[][2] = {
 	{ "Album",              "nie:title"          },
 	{ "Year",               "nie:contentCreated" },
 	{ "copyright",          "nie:copyright"      },
-	{ NULL,                         NULL,                }
+	{ NULL,                 NULL,                }
 };
 
 typedef struct {
@@ -138,7 +129,7 @@ extract_mplayer (const gchar          *uri,
 	argv[8] = g_filename_from_uri (uri, NULL, NULL);
 	argv[9] = NULL;
 
-	if (tracker_spawn (argv, 10, &mplayer, NULL)) {
+	if (tracker_spawn (argv, 10, &mplayer, NULL, NULL)) {
 		GPatternSpec  *pattern_ID_AUDIO_ID;
 		GPatternSpec  *pattern_ID_VIDEO_ID;
 		GPatternSpec  *pattern_ID_AUDIO;
@@ -219,7 +210,10 @@ extract_mplayer (const gchar          *uri,
 
 								equal_char_pos = strchr (next_line, '=');
 
-								data = g_strdup (equal_char_pos + 1);
+								if (g_strcmp0 (info_tags[i][0], "Year") == 0)
+									data = tracker_date_guess (equal_char_pos + 1);
+								else
+									data = g_strdup (equal_char_pos + 1);
 
 								if (data) {
 									if (data[0] != '\0') {
