@@ -1,6 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -21,17 +20,8 @@
  *  Philip Van Hoof <philip@codeminded.be>
  */
 
-#ifndef __TRACKERD_PUSH_H__
-#define __TRACKERD_PUSH_H__
-
-#if !defined (TRACKER_ENABLE_INTERNALS) && !defined (TRACKER_COMPILATION)
-#error "TRACKER_ENABLE_INTERNALS not defined, this must be defined to use tracker's internal functions"
-#endif
-
-#include <glib.h>
-#include <gmodule.h>
-#include <glib-object.h>
-#include <dbus/dbus-glib-bindings.h>
+#ifndef __TRACKER_STORE_PUSH_H__
+#define __TRACKER_STORE_PUSH_H__
 
 G_BEGIN_DECLS
 
@@ -40,4 +30,4 @@ void tracker_push_shutdown (void);
 
 G_END_DECLS
 
-#endif /* __TRACKERD_PUSH_H__ */
+#endif /* __TRACKER_STORE_PUSH_H__ */
