@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,11 +28,11 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <libtracker-client/tracker.h>
-#include <libtracker-common/tracker-type-utils.h>
+#include <libtracker-client/tracker-client.h>
+#include <libtracker-common/tracker-common.h>
 #include <libtracker-miner/tracker-miner.h>
 
-#include "tracker-miner-client.h"
+#include "tracker-status-client.h"
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -44,20 +44,36 @@
 	"\n" \
 	"  http://www.gnu.org/licenses/gpl.txt\n"
 
+static DBusGConnection *connection = NULL;
+static DBusGProxy *proxy = NULL;
+
 static GMainLoop *main_loop;
 static GHashTable *miners_progress;
 static GHashTable *miners_status;
 static gint longest_miner_name_length = 0;
+static gint paused_length = 0;
 
-static gboolean   list_miners_running;
-static gboolean   list_miners_available;
-static gboolean   pause_details;
-static gchar     *miner_name;
-static gchar     *pause_reason;
-static gint       resume_cookie = -1;
-static gboolean   follow;
-static gboolean   detailed;
-static gboolean   print_version;
+static gboolean list_common_statuses;
+static gboolean list_miners_running;
+static gboolean list_miners_available;
+static gboolean pause_details;
+static gchar *miner_name;
+static gchar *pause_reason;
+static gint resume_cookie = -1;
+static gboolean follow;
+static gboolean detailed;
+static gboolean print_version;
+
+/* Make sure our statuses are translated (all from libtracker-miner except one) */
+static const gchar *statuses[7] = {
+	N_("Initializing"),
+	N_("Processing…"),
+	N_("Fetching…"), /* miner/rss */
+	N_("Crawling single directory '%s'"),
+	N_("Crawling recursively directory '%s'"),
+	N_("Paused"),
+	N_("Idle")
+};
 
 static GOptionEntry entries[] = {
 	{ "follow", 'f', 0, G_OPTION_ARG_NONE, &follow,
@@ -68,6 +84,10 @@ static GOptionEntry entries[] = {
 	  N_("Include details with state updates (only applies to --follow)"),
 	  NULL
 	},
+	{ "list-common-statuses", 's', 0, G_OPTION_ARG_NONE, &list_common_statuses,
+	  N_("List common statuses for miners and the store"),
+	  NULL
+	},
 	{ "list-miners-running", 'l', 0, G_OPTION_ARG_NONE, &list_miners_running,
 	  N_("List all miners currently running"),
 	  NULL
@@ -115,6 +135,7 @@ signal_handler (int signo)
 		in_loop = TRUE;
 		g_main_loop_quit (main_loop);
 
+		/* Fall through */
 	default:
 		if (g_strsignal (signo)) {
 			g_print ("\n");
@@ -227,6 +248,7 @@ miner_print_state (TrackerMinerManager *manager,
 	gchar time_str[64];
 	size_t len;
 	struct tm *local_time;
+	gchar *progress_str;
 
 	if (detailed) {
 		now = time ((time_t *) NULL);
@@ -242,31 +264,97 @@ miner_print_state (TrackerMinerManager *manager,
 
 	name = tracker_miner_manager_get_display_name (manager, miner_name);
 
+	if (!is_running) {
+		progress_str = g_strdup_printf ("✗   ");
+	} else if (progress > 0.0 && progress < 1.0) {
+		progress_str = g_strdup_printf ("%-3.0f%%", progress * 100);
+	} else {
+		progress_str = g_strdup_printf ("✓   ");
+	}
+
 	if (is_running) {
-		const gchar *paused;
-		gint paused_len;
-
-		paused = _("PAUSED");
-		paused_len = strlen (paused);
-
-		g_print ("%s  %3.0f%%  %-*.*s %s%-*.*s%s %s %s\n",
+		g_print ("%s  %s  %-*.*s %s%-*.*s%s %s %s\n",
 		         time_str,
-		         progress * 100,
+		         progress_str,
 		         longest_miner_name_length,
 		         longest_miner_name_length,
 		         name,
 		         is_paused ? "(" : " ",
-		         paused_len,
-		         paused_len,
-		         is_paused ? paused : " ",
+		         paused_length,
+		         paused_length,
+		         is_paused ? _("PAUSED") : " ",
 		         is_paused ? ")" : " ",
 		         status ? "-" : "",
-		         status ? status : "");
+		         status ? _(status) : "");
 	} else {
-		g_print ("%s  ---   %s\n",
+		g_print ("%s  %s  %-*.*s  %-*.*s  - %s\n",
 		         time_str,
-		         name);
+		         progress_str,
+		         longest_miner_name_length,
+		         longest_miner_name_length,
+		         name,
+		         paused_length,
+		         paused_length,
+		         " ",
+		         _("Not running or is a disabled plugin"));
 	}
+
+	g_free (progress_str);
+}
+
+static void
+store_print_state (void)
+{
+	GError *error = NULL;
+	gdouble progress;
+	gchar *status;
+	gchar time_str[64];
+	gchar *progress_str;
+
+	org_freedesktop_Tracker1_Status_get_progress (proxy, &progress, &error);
+
+	if (error) {
+		g_critical ("Could not retrieve tracker-store progress: %s", error->message);
+		return;
+	}
+
+	org_freedesktop_Tracker1_Status_get_status (proxy, &status, &error);
+
+	if (error) {
+		g_critical ("Could not retrieve tracker-store status: %s", error->message);
+		return;
+	}
+
+	if (detailed) {
+		struct tm *local_time;
+		time_t now;
+		size_t len;
+
+		now = time ((time_t *) NULL);
+		local_time = localtime (&now);
+		len = strftime (time_str,
+		                sizeof (time_str) - 1,
+		                "%d %b %Y, %H:%M:%S:",
+		                local_time);
+		time_str[len] = '\0';
+	} else {
+		time_str[0] = '\0';
+	}
+
+	if (progress > 0.0 && progress < 1.0) {
+		progress_str = g_strdup_printf ("%-3.0f%%", progress * 100);
+	} else {
+		progress_str = g_strdup_printf ("✓   ");
+	}
+	
+	g_print ("%s  %s  %-*.*s    %s %s\n",
+	         time_str,
+	         progress_str,
+	         longest_miner_name_length + paused_length,
+	         longest_miner_name_length + paused_length,
+	         _("Journal replay"),
+	         status ? "-" : "",
+	         status ? _(status) : "");
 }
 
 static void
@@ -292,6 +380,15 @@ manager_miner_progress_cb (TrackerMinerManager *manager,
 	                      gvalue);
 }
 
+static void
+store_progress_cb (TrackerMinerManager *manager,
+                   const gchar         *status,
+                   gdouble              progress)
+{
+	store_print_state ();
+}
+
+
 static void
 manager_miner_paused_cb (TrackerMinerManager *manager,
                          const gchar         *miner_name)
@@ -332,6 +429,37 @@ miners_progress_destroy_notify (gpointer data)
 	g_slice_free (GValue, value);
 }
 
+static gboolean
+init_store_proxy (void)
+{
+	GError *error = NULL;
+
+	if (connection && proxy) {
+		return TRUE;
+	}
+
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+
+	if (error) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return FALSE;
+	}
+
+	proxy = dbus_g_proxy_new_for_name (connection,
+	                                   "org.freedesktop.Tracker1",
+	                                   "/org/freedesktop/Tracker1/Status",
+	                                   "org.freedesktop.Tracker1.Status");
+
+	dbus_g_proxy_add_signal (proxy,
+	                         "Progress",
+	                         G_TYPE_STRING,
+	                         G_TYPE_DOUBLE,
+	                         G_TYPE_INVALID);
+	return TRUE;
+}
+
 gint
 main (gint argc, gchar *argv[])
 {
@@ -422,6 +550,18 @@ main (gint argc, gchar *argv[])
 	miners_available = tracker_miner_manager_get_available (manager);
 	miners_running = tracker_miner_manager_get_running (manager);
 
+	if (list_common_statuses) {
+		gint i;
+
+		g_print ("%s:\n", _("Common statuses include"));
+
+		for (i = 0; i < G_N_ELEMENTS(statuses); i++) {
+			g_print ("  %s\n", _(statuses[i]));
+		}
+
+		return EXIT_SUCCESS;
+	}
+
 	if (list_miners_available) {
 		gchar *str;
 
@@ -540,7 +680,8 @@ main (gint argc, gchar *argv[])
 		return EXIT_SUCCESS;
 	}
 
-	g_print ("%s:\n", _("Miners"));
+	/* Work out lengths for output spacing */
+	paused_length = strlen (_("PAUSED"));
 
 	for (l = miners_available; l; l = l->next) {
 		const gchar *name;
@@ -549,6 +690,15 @@ main (gint argc, gchar *argv[])
 		longest_miner_name_length = MAX (longest_miner_name_length, strlen (name));
 	}
 
+	/* Display states */
+	g_print ("%s:\n", _("Store"));
+	init_store_proxy ();
+	store_print_state ();
+
+	g_print ("\n");
+
+	g_print ("%s:\n", _("Miners"));
+
 	for (l = miners_available; l; l = l->next) {
 		const gchar *name;
 		gboolean is_running;
@@ -598,6 +748,7 @@ main (gint argc, gchar *argv[])
 	if (!follow) {
 		/* Do nothing further */
 		g_object_unref (client);
+		g_print ("\n");
 		return EXIT_SUCCESS;
 	}
 
@@ -609,6 +760,9 @@ main (gint argc, gchar *argv[])
 	                  G_CALLBACK (manager_miner_paused_cb), NULL);
 	g_signal_connect (manager, "miner-resumed",
 	                  G_CALLBACK (manager_miner_resumed_cb), NULL);
+	dbus_g_proxy_connect_signal (proxy, "Progress",
+	                             G_CALLBACK (store_progress_cb),
+	                             NULL, NULL);
 
 	initialize_signal_handler ();
 
