@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-#include <libtracker-client/tracker.h>
+#include <libtracker-client/tracker-client.h>
 #include <libtracker-common/tracker-common.h>
 
 #define ABOUT	  \
@@ -40,15 +40,16 @@
 	"\n" \
 	"  http://www.gnu.org/licenses/gpl.txt\n"
 
-static gint          limit = 512;
-static gint          offset;
-static gchar       **files;
-static gboolean      or_operator;
-static gchar        *add_tag;
-static gchar        *remove_tag;
-static gboolean     *list;
-static gboolean      show_files;
-static gboolean      print_version;
+static gint limit = 512;
+static gint offset;
+static gchar **files;
+static gboolean or_operator;
+static gchar *add_tag;
+static gchar *remove_tag;
+static gchar *description;
+static gboolean *list;
+static gboolean show_files;
+static gboolean print_version;
 
 static GOptionEntry entries[] = {
 	{ "limit", 'l', 0, G_OPTION_ARG_INT, &limit,
@@ -79,13 +80,17 @@ static GOptionEntry entries[] = {
 	  N_("Delete a tag (if FILEs are omitted, TAG is removed for all files)"),
 	  N_("TAG")
 	},
+	{ "description", 'e', 0, G_OPTION_ARG_STRING, &description,
+	  N_("Description for a tag (this is only used with --add)"),
+	  N_("STRING")
+	},
 	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
 	  N_("Print version"),
 	  NULL
 	},
 	{ G_OPTION_REMAINING, 0,
 	  G_OPTION_FLAG_FILENAME,
-	  G_OPTION_ARG_STRING_ARRAY, &files,
+	  G_OPTION_ARG_FILENAME_ARRAY, &files,
 	  N_("FILE…"),
 	  N_("FILE [FILE…]")},
 	{ NULL }
@@ -148,7 +153,8 @@ get_escaped_sparql_string (const gchar *str)
 static gchar *
 get_fts_string (GStrv    search_words,
                 gboolean use_or_operator,
-                gboolean for_regex)
+                gboolean for_regex,
+                gboolean use_asterisk)
 {
 	GString *fts;
 	gint i, len;
@@ -162,7 +168,10 @@ get_fts_string (GStrv    search_words,
 
 	for (i = 0; i < len; i++) {
 		g_string_append (fts, search_words[i]);
-		g_string_append_c (fts, '*');
+
+                if (use_asterisk) {
+                        g_string_append_c (fts, '*');
+                }
 
 		if (i < len - 1) {
 			if (for_regex) {
@@ -186,6 +195,7 @@ get_fts_string (GStrv    search_words,
 
 static gchar *
 get_filter_string (GStrv        files,
+                   gboolean     files_are_urns,
                    const gchar *tag)
 {
 	GString *filter;
@@ -210,15 +220,11 @@ get_filter_string (GStrv        files,
 	}
 
 	for (i = 0; i < len; i++) {
-		GFile *file;
-		gchar *uri;
-
-		file = g_file_new_for_commandline_arg (files[i]);
-		uri = g_file_get_uri (file);
-		g_object_unref (file);
-
-		g_string_append_printf (filter, "?f = <%s>", uri);
-		g_free (uri);
+		if (files_are_urns) {
+			g_string_append_printf (filter, "?urn = <%s>", files[i]);
+		} else {
+			g_string_append_printf (filter, "?f = \"%s\"", files[i]);
+		}
 
 		if (i < len - 1) {
 			g_string_append (filter, " || ");
@@ -234,6 +240,94 @@ get_filter_string (GStrv        files,
 	return g_string_free (filter, FALSE);
 }
 
+static GStrv
+get_uris (GStrv files)
+{
+	GStrv uris;
+	gint len, i;
+
+	if (!files) {
+		return NULL;
+	}
+
+	len = g_strv_length (files);
+
+	if (len < 1) {
+		return NULL;
+	}
+
+	uris = g_new0 (gchar *, len + 1);
+
+	for (i = 0; files[i]; i++) {
+		GFile *file;
+
+		file = g_file_new_for_commandline_arg (files[i]);
+		uris[i] = g_file_get_uri (file);
+		g_object_unref (file);
+	}
+
+	return uris;
+}
+
+static GPtrArray *
+get_file_urns (TrackerClient *client,
+	       GStrv          uris,
+	       const gchar   *tag)
+{
+	GPtrArray *results;
+	gchar *query, *filter;
+	GError *error = NULL;
+
+	filter = get_filter_string (uris, FALSE, tag);
+	query = g_strdup_printf ("SELECT ?urn ?f "
+	                         "WHERE { "
+	                         "  ?urn "
+	                         "    %s "
+	                         "    nie:url ?f . "
+	                         "  %s "
+	                         "}",
+	                         tag ? "nao:hasTag ?t ; " : "",
+	                         filter);
+
+	results = tracker_resources_sparql_query (client, query, &error);
+
+	g_free (query);
+	g_free (filter);
+
+	if (error) {
+		g_print ("    %s, %s\n",
+		         _("Could not get file URNs"),
+		         error->message);
+		g_error_free (error);
+		return NULL;
+	}
+
+	return results;
+}
+
+static GStrv
+result_to_strv (GPtrArray *result,
+                gint       n_col)
+{
+	GStrv strv;
+	gint i;
+
+	if (!result || result->len == 0) {
+		return NULL;
+	}
+
+	strv = g_new0 (gchar *, result->len + 1);
+
+	for (i = 0; i < result->len; i++) {
+		gchar **row;
+
+		row = g_ptr_array_index (result, i);
+		strv[i] = g_strdup (row[n_col]);
+	}
+
+	return strv;
+}
+
 static void
 get_all_tags_foreach (gpointer value,
                       gpointer user_data)
@@ -244,17 +338,27 @@ get_all_tags_foreach (gpointer value,
 	GStrv data;
 	const gchar *id;
 	const gchar *tag;
+	const gchar *description;
 	gchar *query;
 	gint files, i;
 
 	data = value;
 	client = user_data;
 
-	files = atoi (data[2]);
+	files = atoi (data[3]);
 	id = data[0];
 	tag = data[1];
+	description = data[2];
 
-	g_print ("  %s\n", tag);
+	if (description && *description == '\0') {
+		description = NULL;
+	}
+
+	g_print ("  %s %s%s%s\n", 
+		 tag,
+		 description ? "(" : "",
+		 description ? description : "",
+		 description ? ")" : "");
 
 	g_print ("    %s\n", id);
 
@@ -271,9 +375,10 @@ get_all_tags_foreach (gpointer value,
 	}
 
 	/* Get files associated */
-	query = g_strdup_printf ("SELECT ?urn WHERE {"
+	query = g_strdup_printf ("SELECT ?uri WHERE {"
 	                         "  ?urn a rdfs:Resource; "
-	                         "  nao:hasTag \"%s\" "
+				 "  nie:url ?uri ; "
+	                         "  nao:hasTag \"%s\" . "
 	                         "}",
 	                         id);
 
@@ -312,17 +417,17 @@ get_all_tags (TrackerClient *client,
 	gchar *fts;
 	gchar *query;
 
-	fts = get_fts_string (files, use_or_operator, TRUE);
+	fts = get_fts_string (files, use_or_operator, TRUE, FALSE);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT ?tag ?label COUNT(?urns) AS urns "
+		query = g_strdup_printf ("SELECT ?tag ?label nao:description(?tag) COUNT(?urns) AS urns "
 		                         "WHERE {"
 		                         "  ?tag a nao:Tag ;"
 		                         "  nao:prefLabel ?label ."
 		                         "  OPTIONAL {"
 		                         "     ?urns nao:hasTag ?tag"
 		                         "  } ."
-		                         "  FILTER regex (?label, \"%s\")"
+		                         "  FILTER (?label = \"%s\")"
 		                         "} "
 		                         "GROUP BY ?tag "
 		                         "ORDER BY ASC(?label) "
@@ -332,7 +437,7 @@ get_all_tags (TrackerClient *client,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT ?tag ?label COUNT(?urns) AS urns "
+		query = g_strdup_printf ("SELECT ?tag ?label nao:description(?tag) COUNT(?urns) AS urns "
 		                         "WHERE {"
 		                         "  ?tag a nao:Tag ;"
 		                         "  nao:prefLabel ?label ."
@@ -388,71 +493,91 @@ get_all_tags (TrackerClient *client,
 	return TRUE;
 }
 
+static void
+print_file_report (GPtrArray   *urns,
+                   GStrv        uris,
+                   const gchar *found_msg,
+                   const gchar *not_found_msg)
+{
+	gint i, j;
+
+	if (!urns || !uris) {
+		g_print ("  No files were modified.\n");
+		return;
+	}
+
+	for (i = 0; uris[i]; i++) {
+		gboolean found = FALSE;
+
+		for (j = 0; j < urns->len; j++) {
+			gchar **row;
+
+			row = g_ptr_array_index (urns, j);
+
+			if (g_strcmp0 (row[1], uris[i]) == 0) {
+				found = TRUE;
+				break;
+			}
+		}
+
+		g_print ("  %s: %s\n",
+		         found ? found_msg : not_found_msg,
+		         uris[i]);
+	}
+}
+
 static gboolean
 add_tag_for_urns (TrackerClient *client,
                   GStrv          files,
-                  const gchar   *tag)
+                  const gchar   *tag,
+		  const gchar   *description)
 {
+	GPtrArray *urns = NULL;
 	GError *error = NULL;
-	gchar *filter;
+	GStrv  uris;
 	gchar *tag_escaped;
 	gchar *query;
 
 	tag_escaped = get_escaped_sparql_string (tag);
-	filter = get_filter_string (files, NULL);
 
-	/* First we check if the tag is already set and only add if it
-	 * is, then we add the urns specified to the new tag.
-	 */
-	if (filter) {
+	if (description) {
+		gchar *description_escaped;
 
-		/* We use IE = DO, so we can optimize using nie:isStoredAs instead of
-	 	* nie:url here. Please change this when we change that decision. */
+		description_escaped = get_escaped_sparql_string (description);
 
-		/* Add tag to specific urns */
 		query = g_strdup_printf ("INSERT { "
-		                         "  _:tag a nao:Tag;"
-		                         "  nao:prefLabel %s ."
-		                         "} "
-		                         "WHERE {"
-		                         "  OPTIONAL {"
-		                         "     ?tag a nao:Tag ;"
-		                         "     nao:prefLabel %s"
-		                         "  } ."
-		                         "  FILTER (!bound(?tag)) "
-		                         "} "
-		                         "INSERT { "
-		                         "  ?urn nao:hasTag ?id "
-		                         "} "
-		                         "WHERE {"
-		                         "  ?urn nie:isStoredAs ?f ."
-		                         "  ?id nao:prefLabel %s "
-		                         "  %s "
-		                         "}",
-		                         tag_escaped,
-		                         tag_escaped,
-		                         tag_escaped,
-		                         filter);
+					 "  _:tag a nao:Tag;"
+					 "  nao:prefLabel %s ;"
+					 "  nao:description %s ."
+					 "} "
+					 "WHERE {"
+					 "  OPTIONAL {"
+					 "     ?tag a nao:Tag ;"
+					 "     nao:prefLabel %s ."
+					 "  } ."
+					 "  FILTER (!bound(?tag)) "
+					 "}",
+					 tag_escaped,
+					 description_escaped,
+					 tag_escaped);
+
+		g_free (description_escaped);
 	} else {
-		/* Add tag and do not link it to urns */
 		query = g_strdup_printf ("INSERT { "
-		                         "  _:tag a nao:Tag;"
-		                         "  nao:prefLabel %s ."
-		                         "} "
-		                         "WHERE {"
-		                         "  OPTIONAL {"
-		                         "     ?tag a nao:Tag ;"
-		                         "     nao:prefLabel %s"
-		                         "  } ."
-		                         "  FILTER (!bound(?tag)) "
-		                         "}",
-		                         tag_escaped,
-		                         tag_escaped);
+					 "  _:tag a nao:Tag;"
+					 "  nao:prefLabel %s ."
+					 "} "
+					 "WHERE {"
+					 "  OPTIONAL {"
+					 "     ?tag a nao:Tag ;"
+					 "     nao:prefLabel %s ."
+					 "  } ."
+					 "  FILTER (!bound(?tag)) "
+					 "}",
+					 tag_escaped,
+					 tag_escaped);
 	}
 
-	g_free (tag_escaped);
-	g_free (filter);
-
 	tracker_resources_sparql_update (client, query, &error);
 	g_free (query);
 
@@ -461,6 +586,7 @@ add_tag_for_urns (TrackerClient *client,
 		            _("Could not add tag"),
 		            error->message);
 		g_error_free (error);
+		g_free (tag_escaped);
 
 		return FALSE;
 	}
@@ -468,6 +594,66 @@ add_tag_for_urns (TrackerClient *client,
 	g_print ("%s\n",
 	         _("Tag was added successfully"));
 
+	uris = get_uris (files);
+
+	if (!uris) {
+		/* No URIs to tag */
+		g_free (tag_escaped);
+		return TRUE;
+	}
+
+	urns = get_file_urns (client, uris, NULL);
+
+	/* First we check if the tag is already set and only add if it
+	 * is, then we add the urns specified to the new tag.
+	 */
+	if (urns && urns->len > 0) {
+		GStrv urns_strv;
+		gchar *filter;
+
+		urns_strv = result_to_strv (urns, 0);
+		filter = get_filter_string (urns_strv, TRUE, NULL);
+
+		/* Add tag to specific urns */
+		query = g_strdup_printf ("INSERT { "
+		                         "  ?urn nao:hasTag ?id "
+		                         "} "
+		                         "WHERE {"
+		                         "  ?urn nie:url ?f ."
+		                         "  ?id nao:prefLabel %s "
+		                         "  %s "
+		                         "}",
+		                         tag_escaped,
+		                         filter);
+
+		tracker_resources_sparql_update (client, query, &error);
+		g_strfreev (urns_strv);
+		g_free (filter);
+		g_free (query);
+
+		if (error) {
+			g_printerr ("%s, %s\n",
+			            _("Could not add tag to files"),
+			            error->message);
+			g_error_free (error);
+			g_free (tag_escaped);
+
+			return FALSE;
+		}
+	}
+
+	if (urns) {
+		print_file_report (urns, uris,
+		                   _("Tagged"),
+		                   _("Not tagged, file is not indexed"));
+
+		g_ptr_array_foreach (urns, (GFunc) g_strfreev, NULL);
+		g_ptr_array_free (urns, TRUE);
+	}
+
+	g_strfreev (uris);
+	g_free (tag_escaped);
+
 	return TRUE;
 }
 
@@ -476,16 +662,20 @@ remove_tag_for_urns (TrackerClient *client,
                      GStrv          files,
                      const gchar   *tag)
 {
+	GPtrArray *urns = NULL;
 	GError *error = NULL;
 	gchar *tag_escaped;
 	gchar *query;
+	GStrv uris;
 
 	tag_escaped = get_escaped_sparql_string (tag);
+	uris = get_uris (files);
 
-	if (files && *files) {
+	if (uris && *uris) {
 		GPtrArray *results;
 		gchar *filter;
 		const gchar *urn;
+		GStrv urns_strv;
 
 		/* Get all tags urns */
 		query = g_strdup_printf ("SELECT ?tag "
@@ -504,6 +694,7 @@ remove_tag_for_urns (TrackerClient *client,
 			            error->message);
 			g_error_free (error);
 			g_free (tag_escaped);
+			g_strfreev (uris);
 
 			return FALSE;
 		}
@@ -513,39 +704,40 @@ remove_tag_for_urns (TrackerClient *client,
 			         _("No tags were found by that name"));
 
 			g_free (tag_escaped);
+			g_strfreev (uris);
 
 			return TRUE;
 		}
 
 		urn = * (GStrv) results->pdata[0];
-		filter = get_filter_string (files, urn);
+		urns = get_file_urns (client, uris, urn);
+
+		if (!urns || urns->len == 0) {
+			g_print ("%s\n",
+			         _("None of the files had this tag set"));
+
+			g_strfreev (uris);
+			g_free (tag_escaped);
+
+			return TRUE;
+		}
+
+		urns_strv = result_to_strv (urns, 0);
+		filter = get_filter_string (urns_strv, TRUE, urn);
 
 		query = g_strdup_printf ("DELETE { "
-		                         "  ?f nao:hasTag ?t "
+		                         "  ?urn nao:hasTag ?t "
 		                         "} "
 		                         "WHERE { "
-		                         "  ?f nao:hasTag ?t ."
+		                         "  ?urn nao:hasTag ?t . "
 		                         "  %s "
 		                         "}",
 		                         filter);
 		g_free (filter);
-#if 0
-		filter = get_filter_string (files);
-
-		query = g_strdup_printf ("DELETE {"
-		                         "  <%s> nao:hasTag ?tag "
-		                         "} "
-		                         "WHERE {"
-		                         "  <%s> nao:hasTag ?tag"
-		                         "  FILTER (?tag = <%s>)"
-		                         "}",
-		                         files[0],
-		                         files[0],
-		                         urn);
+		g_strfreev (urns_strv);
 
 		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
 		g_ptr_array_free (results, TRUE);
-#endif
 	} else {
 		/* Remove tag completely */
 		query = g_strdup_printf ("DELETE { "
@@ -567,6 +759,7 @@ remove_tag_for_urns (TrackerClient *client,
 		            _("Could not remove tag"),
 		            error->message);
 		g_error_free (error);
+		g_strfreev (uris);
 
 		return FALSE;
 	}
@@ -574,6 +767,17 @@ remove_tag_for_urns (TrackerClient *client,
 	g_print ("%s\n",
 	         _("Tag was removed successfully"));
 
+	if (urns) {
+		print_file_report (urns, uris,
+		                   _("Untagged"),
+		                   _("File not indexed or already untagged"));
+
+		g_ptr_array_foreach (urns, (GFunc) g_strfreev, NULL);
+		g_ptr_array_free (urns, TRUE);
+	}
+
+	g_strfreev (uris);
+
 	return TRUE;
 }
 
@@ -588,7 +792,7 @@ get_tags_by_file_foreach (gpointer value,
 
 static gboolean
 get_tags_by_file (TrackerClient *client,
-                  const gchar   *file)
+                  const gchar   *uri)
 {
 	GError *error = NULL;
 	GPtrArray *results;
@@ -596,13 +800,13 @@ get_tags_by_file (TrackerClient *client,
 
 	query = g_strdup_printf ("SELECT ?tags ?labels "
 	                         "WHERE {"
-	                         "  <%s>"
-	                         "  nao:hasTag ?tags ."
+	                         "  ?urn nao:hasTag ?tags ;"
+	                         "  nie:url <%s> ."
 	                         "  ?tags a nao:Tag ;"
 	                         "  nao:prefLabel ?labels "
 	                         "} "
 	                         "ORDER BY ASC(?labels)",
-	                         file);
+	                         uri);
 
 	results = tracker_resources_sparql_query (client, query, &error);
 	g_free (query);
@@ -668,6 +872,8 @@ main (int argc, char **argv)
 		failed = _("Add and delete actions can not be used together");
 	} else if (!list && !add_tag && !remove_tag && !files) {
 		failed = _("No arguments were provided");
+	} else if (description && !add_tag) {
+		failed = _("The --description option can only be used with --add");
 	}
 
 	if (failed) {
@@ -705,7 +911,7 @@ main (int argc, char **argv)
 	if (add_tag) {
 		gboolean success;
 
-		success = add_tag_for_urns (client, files, add_tag);
+		success = add_tag_for_urns (client, files, add_tag, description);
 		g_object_unref (client);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
@@ -725,9 +931,18 @@ main (int argc, char **argv)
 		gchar **p;
 
 		for (p = files; *p; p++) {
-			g_print ("<%s>\n", *p);
-			success &= get_tags_by_file (client, *p);
+			GFile *file;
+			gchar *uri;
+			
+			file = g_file_new_for_commandline_arg (*p);
+			uri = g_file_get_uri (file);
+			g_object_unref (file);
+
+			g_print ("%s\n", uri);
+			success &= get_tags_by_file (client, uri);
 			g_print ("\n");
+
+			g_free (uri);
 		}
 
 		g_object_unref (client);
