@@ -239,6 +239,42 @@ writeback_xmp_update_file_metadata (TrackerWritebackFile *wbf,
 			}
 		}
 
+
+		if (g_strcmp0 (row[2], TRACKER_NAO_PREFIX "hasTag") == 0) {
+			GPtrArray *name_array;
+			GError *error = NULL;
+			gchar *query;
+
+			query = g_strdup_printf ("SELECT ?label { "
+			                         "  <%s> nao:prefLabel ?label "
+			                         "}", row[3]);
+
+			name_array = tracker_resources_sparql_query (client, query, &error);
+
+			g_free (query);
+
+			if (name_array && name_array->len > 0) {
+				GStrv name_row;
+
+				name_row = g_ptr_array_index (name_array, 0);
+
+				if (name_row[0]) {
+					if (!keywords) {
+						keywords = g_string_new (name_row[0]);
+					} else {
+						g_string_append_printf (keywords, ", %s", name_row[0]);
+					}
+				}
+			}
+
+			if (name_array) {
+				g_ptr_array_foreach (name_array, (GFunc) g_strfreev, NULL);
+				g_ptr_array_free (name_array, TRUE);
+			}
+
+			g_clear_error (&error);
+		}
+
 		if (g_strcmp0 (row[2], TRACKER_NIE_PREFIX "contentCreated") == 0) {
 			xmp_delete_property (xmp, NS_EXIF, "Date");
 			xmp_set_property (xmp, NS_EXIF, "Date", row[3], 0);
@@ -269,6 +305,12 @@ writeback_xmp_update_file_metadata (TrackerWritebackFile *wbf,
 			}
 		}
 
+#ifdef SET_TYPICAL_CAMERA_FIELDS
+		/* Default we don't do this, we shouldn't overwrite fields that are 
+		 * typically set by the camera itself. What do we know (better) than
+		 * the actual camera did, anyway? Even if the user overwrites them in
+		 * the RDF store ... (does he know what he's doing anyway?) */
+
 		if (g_strcmp0 (row[2], TRACKER_NMM_PREFIX "meteringMode") == 0) {
 
 			xmp_delete_property (xmp, NS_EXIF, "MeteringMode");
@@ -318,13 +360,17 @@ writeback_xmp_update_file_metadata (TrackerWritebackFile *wbf,
 
 			if (g_strcmp0 (row[3], TRACKER_NMM_PREFIX "flash-on") == 0) {
 				/* 0 = Flash did not fire
-				   1 = Flash fired */
+				 * 1 = Flash fired */
 				xmp_set_property (xmp, NS_EXIF, "Flash", "1", 0);
 			} else {
 				xmp_set_property (xmp, NS_EXIF, "Flash", "0", 0);
 			}
 		}
 
+
+		/* TODO: Don't write row[3] as-is here, read xmp_specification.pdf,
+		   page 84 (bottom). */
+
 		if (g_strcmp0 (row[2], TRACKER_NMM_PREFIX "focalLength") == 0) {
 			xmp_delete_property (xmp, NS_EXIF, "FocalLength");
 			xmp_set_property (xmp, NS_EXIF, "FocalLength", row[3], 0);
@@ -345,6 +391,7 @@ writeback_xmp_update_file_metadata (TrackerWritebackFile *wbf,
 			xmp_set_property (xmp, NS_EXIF, "FNumber", row[3], 0);
 		}
 
+
 		if (g_strcmp0 (row[2], TRACKER_NMM_PREFIX "camera") == 0) {
 			gchar *work_on = g_strdup (row[3]);
 			gchar *ptr = strchr (work_on, ' ');
@@ -366,6 +413,7 @@ writeback_xmp_update_file_metadata (TrackerWritebackFile *wbf,
 
 			g_free (work_on);
 		}
+#endif /* SET_TYPICAL_CAMERA_FIELDS */
 
 		if (g_strcmp0 (row[2], TRACKER_MLO_PREFIX "location") == 0 ||
 		    g_strcmp0 (row[2], TRACKER_MLO_PREFIX "city") == 0     ||
